@rem This script assembles each of the declared patches and then applies
@rem them to the Knights of the Round program ROM, generating a 
@rem patched knights.zip in directory output\

@rem =========================================================================
@rem =======================   Add new patches here  =========================
@rem =========================================================================
@rem Adding a new patch consists of two steps:
@rem 
@rem 1. Add patch name to this list
@rem    Each patch is expected to have its own directory of the same name
@rem        inside src\, and also a source code file named main.x68 inside 
@rem        that directory

@set ALL_PATCHES=(hello_world,)
@rem @set ALL_PATCHES=()

@rem 2. Define a variable here with the name mypatchname_OFFSET
@rem        and with value equal to the offset (decimal) where the patch
@rem        should be inserted into the ROM

@set hello_world_OFFSET=477302

@rem =========================================================================

@rem This is needed for variables set inside the for loop to be evaluated late
@setlocal EnableDelayedExpansion
@set MERGED_PROGRAM_ROM_FILE=MERGED_kr_23e_AND_kr_22.bin

@rem The program ROM is split into these two files
@set ROM_KR_23E=kr_23e.8f
@set ROM_KR_22=kr_22.7f

@echo ++++++++ Starting make.bat

@rem Remove old output directory
@call clean.bat

@if not exist original_romset\knights.zip (
    @echo =======================================================================
    @echo = Error: missing original ROM set                                     =
    @echo =                                                                     =
    @echo = Please place original ROM set Knights of the Round (World, 911127^)  =
    @echo =   inside directory original_romset\    (file should be knights.zip^) =
    @echo =======================================================================
    @exit /b
)

@echo ++++++++ Creating directory output\
@mkdir output

@echo ++++++++ Extracting original ROM set to output\modified_romset
@mkdir output\modified_romset
@bin\7-Zip\7z.exe x original_romset\knights.zip -ooutput\modified_romset\

@rem The two program ROMs kr_23e (0 to 0x7FFFFF)
@rem     and kr_22 (0x80000 to 0xFFFFF) must be merged so that patches that
@rem     straddle both can be applied more easily

@rem Note: Some (older) ROM sets have their roms with .rom extension, so we
@rem       need to support those, as well
@if exist output\modified_romset\kr_23e.rom (
    @echo ++++++++ Old kr_23e ROM found, with .rom extension
	@echo ++++++++ Warning: newer emulators will probably complain about missing kr_23e.8f ROM
    @set ROM_KR_23E=kr_23e.rom
)
@if exist output\modified_romset\kr_22.rom (
    @echo ++++++++ Old kr_22 ROM found, with .rom extension
	@echo ++++++++ Warning: newer emulators will probably complain about missing kr_22.7f ROM
    @set ROM_KR_22=kr_22.rom
)

@echo ++++++++ kr_23e rom file: %ROM_KR_23E%
@echo ++++++++ kr_22 rom file: %ROM_KR_22%
@echo ++++++++ merged rom file: %MERGED_PROGRAM_ROM_FILE%


@echo ++++++++ Merging program ROMs
@copy output\modified_romset\%ROM_KR_23E% /B + output\modified_romset\%ROM_KR_22% /B output\modified_romset\%MERGED_PROGRAM_ROM_FILE%

@echo ++++++++ Assembling patches
@mkdir output\patches
@for %%A in %ALL_PATCHES% do @(
	@cd src\%%A
	@rem We're now inside a patch source directory
	
	@echo ++++++++ Assembling patch %%A
	@..\..\bin\LEA\LEA /O=B1 /W=true /Z=false /Q=false /P=%%A main.x68
	
	@echo ++++++++ Copying patch %%A and listing into output\patches\
	@copy /Y Output\* ..\..\output\patches\
	
	@echo ++++++++ Cleaning up assembler leftovers for patch %%A
	@del /F /Q Output\*
	@rmdir Output
	
	@cd ..\..
)

@echo ++++++++ Applying patches to merged program ROM
@for %%A in %ALL_PATCHES% do @(
	@echo ++++++++ Byteswapping patch %%A
	@bin\byteswap.exe output\patches\%%A.bin
	
	@rem ! is used here instead of % to delay variable evaluation until the loop is actually entered
	@echo ++++++++ Applying patch %%A
	@set /A OFFSET=%%A_OFFSET
	@bin\binpatch.exe output\modified_romset\%MERGED_PROGRAM_ROM_FILE% !OFFSET! output\patches\%%A.bin
)

@rem Once all patches have been applied to merged program ROM, we split it
@rem back into the two ROM files

@echo ++++++++ Splitting merged program ROM back into separate ROM files
@del /F /Q output\modified_romset\%ROM_KR_23E%
@del /F /Q output\modified_romset\%ROM_KR_22%
@bin\binsplit.exe output\modified_romset\%MERGED_PROGRAM_ROM_FILE% 524288
@ren output\modified_romset\%MERGED_PROGRAM_ROM_FILE%_1 %ROM_KR_23E%
@ren output\modified_romset\%MERGED_PROGRAM_ROM_FILE%_2 %ROM_KR_22%

@echo ++++++++ Deleting merged program ROM
@del /F /Q output\modified_romset\%MERGED_PROGRAM_ROM_FILE%

@echo ++++++++ Zipping modified ROM set
@cd output\modified_romset\
@..\..\bin\7-Zip\7z.exe a ..\knights.zip *

@echo ++++++++ Done

@echo =======================================================================
@echo =                                                                     =
@echo = Find the patched ROM set at output\knights.zip                      =
@echo =                                                                     =
@echo = Note: emulators will likely complain that the checksum is incorrect =
@echo =       this is normal, since ROMs have been modified                 =
@echo =======================================================================
